package com.semarchy.mdm.client.samples;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import com.semarchy.mdm.client.IDataEditionConnection;
import com.semarchy.mdm.client.IMDMConnection;
import com.semarchy.mdm.client.MDMConnectionFactory;
import com.semarchy.mdm.runtime.commons.PageCriteria;
import com.semarchy.mdm.runtime.data.DataQuery;
import com.semarchy.mdm.runtime.data.IDataRow;
import com.semarchy.mdm.runtime.data.IEntityRow;
import com.semarchy.mdm.runtime.model.dataaccess.IDataAccessModel;
import com.semarchy.mdm.runtime.model.dataaccess.IView;
import com.semarchy.mdm.runtime.model.logical.IEntity;
import com.semarchy.mdm.runtime.model.logical.IModel;

public class Sample3_QueryEmployees {

	public static void main(String[] args) {
		IMDMConnection conn = MDMConnectionFactory.connect(
				SamplesConstants.SEMARCHY_URL, SamplesConstants.USERNAME,
				SamplesConstants.PASSWORD);

		IDataEditionConnection dataEditionConnection = conn
				.createDataEditionConnection(SamplesConstants.GS_DATAEDITION);

		IModel model = dataEditionConnection.getLogicalModel();
		IDataAccessModel dataAccessModel = dataEditionConnection
				.getDataAccessModel();

		IEntity employeeEntity = model.getEntity("Employee");
		IView employeeGoldenView = dataAccessModel
				.getGoldenView(employeeEntity);

		listMostRecentlyHiredEmployees(dataEditionConnection,
				employeeGoldenView);

		countEmployeesPerCostCenter(dataEditionConnection, employeeGoldenView);

	}

	private static void countEmployeesPerCostCenter(
			IDataEditionConnection dataEditionConnection,
			IView employeeGoldenView) {
		DataQuery qry = new DataQuery(employeeGoldenView);
		String costCenterName = "CostCenter.CostCenterName";
		qry.addSelectExpression(costCenterName);
		String countExpression = "count(1)";
		qry.addSelectExpression(countExpression);
		qry.addGroupByExpression(costCenterName);

		qry.addOrderByCriteria(countExpression, true);
		qry.setPageCriteria(new PageCriteria(0, 5));
		List<IDataRow> rows = dataEditionConnection.getDataAccessService()
				.queryRows(qry);
		System.out.println("List of " + qry.getPageCriteria().getPageSize()
				+ " cost centers having the more employees:");
		for (IDataRow dataRow : rows) {
			Long count = (Long) dataRow.getValue(countExpression);
			System.out.println(count + " employees in cost center "
					+ dataRow.getValue(costCenterName));
		}
	}

	private static void listMostRecentlyHiredEmployees(
			IDataEditionConnection dataEditionConnection,
			IView employeeGoldenView) {
		DataQuery qry = new DataQuery(employeeGoldenView);
		qry.addSelectExpression("FirstName");
		qry.addSelectExpression("LastName");
		qry.addSelectExpression("HireDate");

		qry.addOrderByCriteria("HireDate", true);
		qry.setPageCriteria(new PageCriteria(0, 20));
		List<IEntityRow> rows = dataEditionConnection.getDataAccessService()
				.queryEntityRows(qry);
		System.out.println("List of " + qry.getPageCriteria().getPageSize()
				+ " most recently hired employees:");
		for (IEntityRow dataRow : rows) {
			String firstName = (String) dataRow.getValue("FirstName");
			String lastName = (String) dataRow.getValue("LastName");
			Date hireDate = (Date) dataRow.getValue("HireDate");
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");
			System.out.println("Employee " + firstName + " " + lastName
					+ " was hired on " + dateFormat.format(hireDate));
		}
	}
}
